/*
 * CG Madness - a Marble Madness clone
 * Copyright (C) 2007  Sven Reinck <sreinck@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "BlockingQueue.hpp"

#include "Synchronized.hpp"

template<class T>
inline BlockingQueue<T>::BlockingQueue() :
	mEmpty(mLock)
{
  // empty
}

template<class T>
inline BlockingQueue<T>::~BlockingQueue()
{
  // empty
}

template<class T>
inline bool BlockingQueue<T>::isEmpty() const
{
	Synchronized sync(mLock);
	
	return mQueue.empty();
}

template<class T>
inline void BlockingQueue<T>::push(const T& item)
{
	Synchronized sync(mLock);
	
	mQueue.push(item);
	
	mEmpty.signal();
}

template<class T>
inline T BlockingQueue<T>::pop()
{
	Synchronized sync(mLock);

	if (mQueue.empty())
	{
		mEmpty.wait();
	}
	
	T item = mQueue.front();
	mQueue.pop();
	
	return item;
}
